@@ -31,6 +31,7 @@
 #include	"rcache.h"
 #include	"save_global.h"
 #include	"mx_format.h"
+#include	"win_flame.h"
 
 typedef struct res_matrix_t {
 	Q_HEADER		h;
@@ -73,6 +74,7 @@ res_matrix_trigger(int type,void * data);
 XLISP_ENV * resource_matrix_env;
 XLISP_ENV * resource_matrix_gmxEnv;
 SEM res_matrix_lock;
+extern SEM res_lock;
 
 void
 init_resource_matrix()
@@ -130,6 +132,15 @@ int
 insert_resource_matrix(RESOURCE * r,int w_flag)
 {
 RES_MATRIX_T * t;
+
+
+	lock_task(res_lock);
+	if ( r->h.mtx_loading ) {
+		unlock_task(res_lock,"insert_resource_matrix");
+		return 0;
+	}
+	r->h.mtx_loading = 1;
+	unlock_task(res_lock,"insert_resource_matrix");
 	t = new_queue_node(sizeof(*t));
 	t->ret = 0;
 	t->r = r;
@@ -144,6 +155,7 @@ insert_res_matrix_read_net(MATRIX_TOKEN * tk)
 {
 RESOURCE * r;
 RES_MATRIX_T * t;
+
 	r = tk->process_node->matrix->open_work;
 	t = new_queue_node(sizeof(*t));
 	t->loop_no = r->pr64.loop_no;
@@ -209,12 +221,12 @@ RESOURCE * r;
 		}
 		else {
 			if ( t->aboat_flag ) {
-				matrix_token_error(t->token);
 				t->token->work = 0;
 				t->token = 0;
+				matrix_token_error(t->token);
 				d_f_ree(t);
 				gc_pop(0,0);
-				break;
+				continue;
 			}
 			r = t->token->process_node->matrix->open_work;
 			sym = n_get_symbol("Get");
@@ -247,6 +259,10 @@ RESOURCE * r;
 }
 
 
+#define LOCK_CNT 5
+MATRIX * debug_target_matrix;
+int lock_cnt;
+
 void
 res_matrix_task_after(TKEY d)
 {
@@ -262,6 +278,7 @@ RS_BUF b;
 L_CHAR * filename;
 XL_SEXP * ret;
 int i;
+RESOURCE * r;
 
 	que = (SYS_QUEUE *)GET_TKEY(d);
 	key = touch_qkey(que);
@@ -289,6 +306,7 @@ int i;
 				ss_printf("ERROR=");
 				print_sexp(s_stdout,t->ret,0);
 				ss_printf("\n");
+				insert_lw_service_code_flag_set(t->r,t->ret);
 				goto err2;
 			}
 			cmd = get_el_by_symbol(t->ret,l_string(std_cm,"gmxStatus"),0);
@@ -349,28 +367,50 @@ int i;
 			}
 		err2:
 			;
+			t->r->h.mtx_loading = 0;
 			d_f_ree(t);
+			lock_task(res_lock);
+			unlock_task(res_lock,"matrix");
 		}
 		else {
 			if ( t->aboat_flag )
 				goto err;
-			if ( get_type(t->ret) == XLT_ERROR )
+			if ( get_type(t->ret) == XLT_ERROR ) {
+				r = (RESOURCE*)t->token->process_node->matrix->open_work;
+				insert_lw_service_code_flag_set(r,t->ret);
 				goto err;
+			}
+
 			if ( decode_matrix_node(t->token->process_node,t->ret) < 0 )
 				goto err;
-			insert_matrix_access(t->token,matrix_exec_cal);
-			goto finish;
-		err:
-			matrix_token_error(t->token);
-		finish:
+
+			debug_target_matrix = t->token->process_node->matrix;
 			lock_task(res_matrix_lock);
 			t->token->work = 0;
 			t->token->aboat_func = 0;
+			insert_matrix_access(t->token,matrix_exec_cal);
 			d_f_ree(t);
 			unlock_task(res_matrix_lock,"res_matrix");
+			goto finish1;
+		err:
+			lock_task(res_matrix_lock);
+			t->token->work = 0;
+			t->token->aboat_func = 0;
+			matrix_token_error(t->token);
+			d_f_ree(t);
+			unlock_task(res_matrix_lock,"res_matrix");
+		finish1:
 			;
 		}
 		gc_pop(0,0);
+/*
+if ( lock_cnt > LOCK_CNT ) {
+ss_printf("LOCK\n");
+for( ; ; )
+sleep_sec(1);
+}
+else lock_cnt ++;
+*/
 	}
 
 	release_qkey(que,key);
