@@ -297,18 +297,6 @@ RESOURCE * r;
 			&r,env,s,a,0));
 }
 
-void
-draw_map()
-{
-	er_panic("draw_map(1)");
-}
-
-void
-get_point_map()
-{
-	er_panic("get_point_map(1)");
-}
-
 void
 lock_map()
 {
@@ -357,7 +345,7 @@ ok2:
 	d_f_ree(m1);
 }
 
-
+/*
 REAL1
 map_radius(MAP_PARAMETER * mp,GB_POINT c,REAL1 radius)
 {
@@ -414,7 +402,7 @@ retry:
 	unlock_task(map_lock,"map_radius");
 	return rr;
 }
-
+*/
 
 int test_ff;
 
@@ -436,7 +424,7 @@ GB_POINT ret;
 	for ( tt = &mp->opt.tri.tri_list ; *tt ; tt = &(*tt)->next ) {
 		if ( inside_rect(&(*tt)->minrect,p) == 0 )
 			continue;
-		if ( inside_triangle((*tt)->ptp,p) == 0 )
+		if ( inside_triangle_arg(&(*tt)->trg_w,(*tt)->ptp,p) == 0 )
 			goto ok;
 	}
 /*
@@ -444,9 +432,7 @@ GB_POINT ret;
 		goto ok_outside;
 */
 	if ( mp->opt.tri.sts == TS_FIX )
-{ss_printf("FIX\n");
 		goto ok_outside;
-}
 	*res = -1;
 	return p;
 ok:
@@ -479,11 +465,63 @@ GB_POINT ret;
 }
 
 
+TRIANGLE *
+check_in_rect_triangle(MAP_PARAMETER * mp,GB_POINT * p,REAL1 * reso,int size,GB_RECT * rct)
+{
+TRIANGLE * ret;
+int i;
+GB_RECT rr;
+TRIANGLE **tt;
+GB_POINT pp;
+	ret = 0;
+	if ( rct ) 
+		rr = *rct;
+	else {
+		rr.tl.x = rr.tl.y = 0;
+		rr.br.x = rr.br.y = -1;
+		for ( i = 0 ; i < size ; i ++ )
+			insert_rect(&rr,p[i]);
+	}
+
+	for ( tt = &mp->opt.tri.tri_list ; *tt ; tt = &(*tt)->next ) {
+		if ( inside_rect(&(*tt)->minrect,rr.tl) == 0 )
+			continue;
+		if ( inside_rect(&(*tt)->minrect,rr.br) == 0 )
+			continue;
+		if ( inside_triangle_arg(&(*tt)->trg_w,(*tt)->ptp,rr.tl) )
+			continue;
+		if ( inside_triangle_arg(&(*tt)->trg_w,(*tt)->ptp,rr.br) )
+			return 0;
+		pp.x = rr.tl.x;
+		pp.y = rr.br.y;
+		if ( inside_triangle_arg(&(*tt)->trg_w,(*tt)->ptp,pp) )
+			return 0;
+		pp.x = rr.br.x;
+		pp.y = rr.tl.y;
+		if ( inside_triangle_arg(&(*tt)->trg_w,(*tt)->ptp,pp) )
+			return 0;
+		goto ok;
+	}
+	return 0;
+ok:
+	ret = *tt;
+	*tt = ret->next;
+	ret->next = mp->opt.tri.tri_list;
+	mp->opt.tri.tri_list = ret;
+	return ret;
+}
+
+
 void
-map_conv_reverse(RESOURCE * mp,GB_POINT * p,REAL1 * reso,int size)
+xx_map_conv_reverse(RESOURCE * mp,MAP_CONVERT_WORK *w,GB_POINT * p,REAL1 * reso,int size,char * file,int line)
 {
 int i;
 GB_POINT mv;
+/*
+TRIANGLE ** tt,*t;
+GB_POINT pp;
+*/
+TRIANGLE * tr;
 
 	if ( mp == 0 )
 		return;
@@ -517,12 +555,101 @@ GB_POINT mv;
 		break;
 	case MT_TRIANGLE:
 		lock_task(map_lock);
-		for ( i = 0 ; i < size ; i ++ ) {
-			if ( reso[i] < 0 )
+/*		if ( mp->map.param_reverse.opt.tri.sts == TS_FIX ) {
+			for ( i = 0 ; i < size  ; i ++ ) {
+				if ( reso[i] < 0 )
+					continue;
+				pp = p[i];
+				if ( inside_rect(&mp->map.param_reverse.opt.tri.minrect,pp) == 0 ) {
+					reso[i] *= mp->map.param_reverse.opt.tri.resolution_rate;
+					p[i] = caffen2d(&mp->map.param_reverse.opt.tri.a,pp);
+					continue;
+				}
+
+				for ( tt = &mp->map.param_reverse.opt.tri.tri_list ; *tt ; tt = &(*tt)->next ) {
+					if ( inside_rect(&(*tt)->minrect,pp) == 0 )
+						continue;
+					if ( inside_triangle_arg(&(*tt)->trg_w,(*tt)->ptp,pp) == 0 )
+						goto ok1;
+				}
+				reso[i] = -1;
 				continue;
-			p[i] = _map_conv_triangle(&reso[i],
-				&mp->map.param_reverse,p[i]);
+			ok1:
+				t = *tt;
+				if ( t != mp->map.param_reverse.opt.tri.tri_list ) {
+					*tt = t->next;
+					t->next = mp->map.param_reverse.opt.tri.tri_list;
+					mp->map.param_reverse.opt.tri.tri_list = t;
+				}
+				reso[i] *= t->resolution_rate;
+				p[i] = caffen2d(&t->a,pp);
+
+			}
 		}
+		else {
+			for ( i = 0 ; i < size  ; i ++ ) {
+				if ( reso[i] < 0 )
+					continue;
+				pp = p[i];
+
+				t = mp->map.param_reverse.opt.tri.tri_list;
+				if ( inside_rect(&t->minrect,pp) &&
+						(inside_triangle_arg(&t->trg_w,t->ptp,pp) == 0) )
+					goto ok3;
+				for ( tt = &mp->map.param_reverse.opt.tri.tri_list->next ; *tt ; tt = &(*tt)->next ) {
+					if ( inside_rect(&(*tt)->minrect,pp) == 0 )
+						continue;
+					if ( inside_triangle_arg(&(*tt)->trg_w,(*tt)->ptp,pp) == 0 )
+						goto ok2;
+				}
+				reso[i] = -1;
+				continue;
+			ok2:
+				t = *tt;
+				if ( t != mp->map.param_reverse.opt.tri.tri_list ) {
+					*tt = t->next;
+					t->next = mp->map.param_reverse.opt.tri.tri_list;
+					mp->map.param_reverse.opt.tri.tri_list = t;
+				}
+			ok3:
+				reso[i] *= t->resolution_rate;
+				p[i] = caffen2d(&t->a,pp);
+			}
+		}
+*/
+
+//ss_printf("a %s %i\n",file,line);
+		if ( w && w->sr ) {
+			if ( w->sr_len != 1 )
+				goto normal;
+			tr = check_in_rect_triangle(&mp->map.param_reverse,p,reso,size,w->sr);
+			if ( tr == 0 )
+				goto normal;
+//ss_printf("==== OPT\n");
+			goto opt;
+		}
+		else if ( (tr = check_in_rect_triangle(&mp->map.param_reverse,p,reso,size,0)) != 0 ) {
+		opt:
+//ss_printf("b\n");
+			for ( i = 0 ; i < size ; i ++ ) {
+				if ( reso[i] < 0 )
+					continue;
+				p[i] = caffen2d(&tr->a,p[i]);
+				reso[i] *= tr->resolution_rate;
+			}
+//ss_printf("c\n");
+		}
+		else {
+		normal:
+			for ( i = 0 ; i < size ; i ++ ) {
+				if ( reso[i] < 0 )
+					continue;
+				p[i] = _map_conv_triangle(&reso[i],
+					&mp->map.param_reverse,p[i]);
+			}
+//ss_printf("d\n");
+		}
+
 		unlock_task(map_lock,"map_conv_reverse");
 		break;
 	default:
@@ -745,7 +872,7 @@ _map_from_flame_trace(MAP_HISTORY * mh,GB_POINT * ptr,REAL1 * reso,int size,int
 				map_conv_forward(mh->d.map,ptr,reso,size);
 				break;
 			case MHD_REVERSE:
-				map_conv_reverse(mh->d.map,ptr,reso,size);
+				map_conv_reverse(mh->d.map,0,ptr,reso,size);
 			}
 			break;
 		case MHT_LINEAR:
@@ -802,7 +929,7 @@ map_from_resource(MAP_HISTORY * mh,GB_POINT * ptr,REAL1 * reso,int size)
 		case MHT_MAP:
 			switch ( mh->dir ) {
 			case MHD_FORWARD:
-				map_conv_reverse(mh->d.map,ptr,reso,size);
+				map_conv_reverse(mh->d.map,0,ptr,reso,size);
 				break;
 			case MHD_REVERSE:
 				map_conv_forward(mh->d.map,ptr,reso,size);
@@ -1483,11 +1610,57 @@ AFFEN2D a;
 	return mh;
 }
 
+MAP_HISTORY *
+optimize_rf(MAP_HISTORY * mh)
+{
+MAP_HISTORY ** mhp,*mh1,*mh2;
+int f;
+	f = 1;
+	for ( ; f ; ) {
+		f = 0;
+		for ( mhp = &mh ; *mhp ;  ) {
+			mh1 = *mhp;
+			mh2 = mh1->next;
+			if ( mh2 == 0 )
+				break;
+			if ( mh1->type != MHT_MAP || mh2->type != MHT_MAP )
+				goto next;
+			if ( mh1->d.map != mh2->d.map )
+				goto next;
+			if ( mh1->dir == mh2->dir )
+				goto next;
+			*mhp = mh2->next;
+			mh1->next = 0;
+			mh2->next = 0;
+			free_mh(mh1);
+			free_mh(mh2);
+			f = 1;
+			continue;
+		next:
+			mhp = &(*mhp)->next;
+		}
+	}
+	return mh;
+}
+
 MAP_HISTORY *
 optimize_mh(MAP_HISTORY * mh)
 {
+	mh = optimize_rf(mh);
 	mh = optimize_mh_identical(mh);
 	mh = optimize_mh_linear(mh);
 	return mh;
 }
 
+int
+map_history_length(MAP_HISTORY * mh)
+{
+int ret;
+
+	ret = 0;
+	for ( ; mh ; mh = mh->next , ret ++ );
+	return ret;
+}
+
+
+
