@@ -42,7 +42,7 @@ typedef struct indicate_t {
 extern RESOURCE ** res_entry_hash_table;
 extern SEM irq_lock,in_lock,iom_lock;
 
-void indicate_info_task();
+void indicate_info_task(TKEY);
 
 
 void
@@ -458,7 +458,7 @@ GF_STATUS * gfs;
 	wf_init_status(&gfs->sts);
 	gfs->sts.flags = SF_REFERENCE;
 	gfs->sts.reference = ll_copy_str(url);
-	new_tick((void(*)())_click_event_tick,0,(int)gfs);
+	new_tick((void(*)(int))_click_event_tick,0,(int)gfs);
 }
 
 void
@@ -470,7 +470,7 @@ GF_STATUS * gfs;
 	wf_init_status(&gfs->sts);
 	gfs->sts.flags = SF_INFOLIST;
 	gfs->sts.infolist = _copy_infolist(gf);
-	new_tick((void(*)())_click_event_tick,0,(int)gfs);
+	new_tick((void(*)(int))_click_event_tick,0,(int)gfs);
 }
 
 void
@@ -481,7 +481,7 @@ GF_STATUS * gfs;
 	gfs->gf = gf;
 	wf_init_status(&gfs->sts);
 	gfs->sts.flags = SF_ERROR;
-	new_tick((void(*)())_click_event_tick,0,(int)&gfs);
+	new_tick((void(*)(int))_click_event_tick,0,(int)&gfs);
 }
 
 
