@@ -250,7 +250,7 @@ retry:
 		goto end;
 	if ( wf->dr_time && wf->data_request && wf->dr_time < get_xltime() ) {
 	void dr_tick(int d);
-//ss_printf("DIRTY SYSTEM\n");
+ss_printf("DIRTY SYSTEM\n");
 		wf_lock(gf);
 		win_flame_dirty(gf,wf->id,0,WFF_DIRTY,0);
 		if ( wf->dr_interval >= 120 ) {
@@ -259,7 +259,7 @@ retry:
 		}
 		else {
 			wf->dr_time = get_xltime() + wf->dr_interval;
-			new_tick((void(*)())dr_tick,-wf->dr_interval-1,(int)gf);
+			new_tick((void(*)(int))dr_tick,-wf->dr_interval-1,(int)gf);
 			wf->dr_interval *= 2;
 		}
 		wf_unlock(gf);
