@@ -465,6 +465,8 @@ int width,x_offset,w_len;
 		wf->pixels = pixels;
 		break;
 	case WFT_PLOT:
+		if ( wf->pixels )
+			d_f_ree(wf->pixels);
 		wf->pixels = d_alloc(gf->win_width * gf->win_height 
 				* sizeof(long));
 		for ( i = 0 ; i < gf->win_width * gf->win_height ; i ++ )
@@ -540,7 +542,7 @@ GB_POINT * pt_list;
 REAL1 * pt_reso;
 int w,h;
 MAP_HISTORY * mh;
-WIN_FLAME * td, * _wf;
+WIN_FLAME * _wf;
 SURP_SET surp;
 int id;
 int ci_ret;
@@ -592,9 +594,11 @@ int ci_ret;
 		wf_unlock(gf);
 		return -2;
 	}
+/*
 	for ( td = wf->next ; td ; td = td->next )
 		if ( td->draw->c.ov->h.filter == ov_default )
 			break;
+*/
 	for ( xx = r->tl.x ; xx < r->br.x ; xx ++ )
 		for ( yy = r->tl.y ; yy < r->br.y ; yy ++ ) {
 			if ( !(wf->pixels[yy*gf->win_width + xx]&C_DIRTY) )
