@@ -27,7 +27,7 @@
 #endif
 #include	<fcntl.h>
 
-extern int (*__log_vprintf)();
+extern int (*__log_vprintf)(int,int,void *,char*,va_list);
 int
 log_vprintf(int level,int layer,void*,char * fmt,va_list p);
 int log_printf(int level,int layer,void*,char * fmt,...);
