@@ -541,6 +541,7 @@ typedef struct mx_cache_param {
 	/* vector read write param */
 	
 	int			vector_len;
+	unsigned char *		vector_valid;
 } MX_CACHE_PARAM;
 
 typedef struct mx_dim_code_list {
@@ -614,6 +615,9 @@ typedef struct mx_sb_list {
 
 #define ME_EXIT			(-125)
 
+#define ME_INVALID_PARAM	(-126)
+#define ME_CANNOT_LOCK		(-127)
+
 /* xlerror application code */
 
 #define XL_AC_FILE		1
@@ -870,6 +874,7 @@ void matrix_exec_cal(MATRIX_TOKEN * t);
 void close_all_matrix();
 void token_check(MATRIX_TOKEN * t);
 void allset_dim_code_index(MATRIX_NODE * n,int dirty);
+void vecset_dim_code_index(MATRIX_NODE * n,char*,int dirty);
 MATRIX_NODE *
 get_matrix_node_wait(int * errp,MATRIX * m,INTEGER64 * dim_code,
 			int access);
@@ -1002,6 +1007,10 @@ void finish_file_access(MATRIX * m,int);
 void finish_file_access_with_wait(MATRIX * m,int wait_key);
 MX_STRUCT_BLOCK * 
 get_channel_struct_block(MATRIX_NODE * n,int ch);
+int
+mx_scale_of_dim_code(INTEGER64 * start,INTEGER64 * last,MATRIX * m);
+int
+matrix_copy(MATRIX * dest,MATRIX * src,int * ch_list,int copy_flags);
 
 
 #endif
