@@ -38,30 +38,31 @@
 #define WFF_BASE		0x00000300
 #define WFF_BASE_CURRENT	0x00000100
 #define WFF_BASE_LOCK		0x00000200
+#define WFF_BASE_P_LOCK		0x00000400
 
 #define WFF_BASE_OFFSET		8
 #define WFF_HIDE_OFFSET		7
 
-#define WFF_DRAW_REQUEST	0x00000400
+#define WFF_DRAW_REQUEST	0x00000800
 
-#define WFF_DIRTY		0x00003800
-#define WFF_LUSTER_DIRTY	0x00000800
-#define WFF_POLY_DIRTY		0x00001000
-#define WFF_PLOT_DIRTY		0x00002000
+#define WFF_DIRTY		0x00007000
+#define WFF_LUSTER_DIRTY	0x00001000
+#define WFF_POLY_DIRTY		0x00002000
+#define WFF_PLOT_DIRTY		0x00004000
 
 
-#define WFF_LUSTER		0x00004000
-#define WFF_POLY		0x00008000
-#define WFF_PLOT		0x00010000
+#define WFF_LUSTER		0x00008000
+#define WFF_POLY		0x00010000
+#define WFF_PLOT		0x00020000
 
-#define WFF_LS_DIRTY		0x00020000
+#define WFF_LS_DIRTY		0x00040000
 
 #define WFF_TO_SMALL		0x00100000
 #define WFF_TO_STD		0x00200000
 
 #define WFF_HIDE_TIME_DISABLE	0x00400000
 
-#define WFF_COPY_FLAGS		(WFF_HIDE|WFF_BASE_LOCK)
+#define WFF_COPY_FLAGS		(WFF_HIDE|WFF_BASE_LOCK|WFF_BASE_P_LOCK)
 
 
 /* for drawing */
@@ -632,5 +633,9 @@ GV_RESOURCE_LIST *
 search_resource_list(GBVIEW_FLAME * gf,RESOURCE *r,L_CHAR *,int create_flag);
 GV_RESOURCE_LIST *
 _search_resource_list(GBVIEW_FLAME * gf,RESOURCE *r,L_CHAR * ,int create_flag);
+void
+wf_auth_information(RESOURCE * r);
+void
+insert_lw_service_code_flag_set(RESOURCE * r,XL_SEXP * s);
 
 #endif
