@@ -36,7 +36,7 @@ fs_raw_write64(PDB64 * p,void * d,int flags);
 void change_endian_freelist64_to_net(PN64_FREELIST * f);
 void change_endian_freelist64_to_host(PN64_FREELIST * f);
 void free_freelist64(FREELIST64 * f);
-void load_freelist64(PDB64 * p,PN64_FREELIST * f,U_INTEGER64 fofs);
+int load_freelist64(PDB64 * p,PN64_FREELIST * f,U_INTEGER64 fofs);
 void save_freelist64(PDB64 * p,PN64_FREELIST * f,U_INTEGER64 fofs);
 FREELIST64 *
 next_freelist64(PDB64 * p,FREELIST64 * f);
@@ -190,7 +190,7 @@ er_panic("A");
 	case DT_DELETE:
 		d.fofs = fofs;
 		d.size = size;
-		a = avt_delete(&p->pdb64_debug_root,dp,pdb64_debug_cmp);
+		a = avt_delete(&p->pdb64_debug_root,&d,pdb64_debug_cmp);
 		if ( a ) {
 			d_f_ree(a->data);
 			d_f_ree(a);
@@ -427,24 +427,24 @@ length,fofs);
 
 		er = u_lseek64(p->fid,fofs,SEEK_SET);
 		if ( er < 0 )
-			er_panic("get_file_record64(1)");
+			return 0;
 		er = u_read(p->fid,&ph,sizeof(PN_HEADER));
 
 // ph.h16.type NET
 
 		if ( er < sizeof(PN_HEADER) )
-			er_panic("get_file_record64(2)");
+			return 0;
 		h16_size = ph.h16.size;
 		change_endian(h16_size);
 		if ( h16_size < sizeof(PN_HEADER) ) {
 			ss_printf("%i %x %x " I64_FORMAT "\n",(int)h16_size,(int)ph.h16.size,(int)ph.h16.type,(INTEGER64)fofs);
-		  	er_panic("data error");
+			return 0;
 		}
 		switch ( h16_size ) {
 		case PHYS_SIZE32:
 			er = u_read(p->fid,&ph.h32.size,sizeof(int));
 			if ( er < sizeof(int) )
-				er_panic("get_file_record64(2)");
+				return 0;
 			change_endian_phys_header64_to_host(&ph);
 // ph.h16.type HOST
 			h_size = sizeof(ph.h32);
@@ -454,7 +454,7 @@ length,fofs);
 			er = u_read(p->fid,&ph.h64.size,
 					sizeof(INTEGER64));
 			if ( er < sizeof(INTEGER64) )
-				er_panic("get_file_record64(2)");
+				return 0;
 			change_endian_phys_header64_to_host(&ph);
 // ph.h16.type HOST
 			h_size = sizeof(ph.h64);
@@ -493,7 +493,7 @@ length,fofs);
 		else	_s = size;
 		er = u_read(p->fid,h+1,_s);
 		if ( er < _s )
-			er_panic("get_file_record(3)");
+			return 0;
 		ptr += er;
 		size -= er;
 	}
@@ -603,12 +603,14 @@ flush_filespace64(PDB64 * p)
 }
 
 
-void
+int
 load_freelist64(PDB64 * p,PN64_FREELIST * f,U_INTEGER64 fofs)
 {
 PN64_FREELIST * _f;
 
 	_f = get_file_record64(&fofs,p,0,sizeof(PN64_FREELIST));
+	if ( _f == 0 )
+		return -1;
 	change_endian_freelist64_to_host(_f);
 
 if ( fofs == 44 )
@@ -616,6 +618,7 @@ ss_printf("LOAD FREELIST64\n");
 
 	*f = *_f;
 	d_f_ree(_f);
+	return 0;
 }
 
 void
@@ -975,12 +978,12 @@ U_INTEGER64 length;
 		if ( fp->fofs >= length ) {
 			fprintf(stderr,"prv" I64_FORMAT " ofs" I64_FORMAT " typ%x siz" I64_FORMAT "\n",
 				prev,fp->fofs,fp->f.h.type,fp->f.h.size);
-			er_panic("check_freelist(0)");
+			er_panic("check_freelist64(0)");
 		}
 		if ( prev && fp->f.h.type != PNT_FREELIST_NODE ) {
 			fprintf(stderr,"prv" I64_FORMAT " ofs" I64_FORMAT " typ%x siz" I64_FORMAT "\n",
 				prev,fp->fofs,fp->f.h.type,fp->f.h.size);
-			er_panic("check_freelist(1)");
+			er_panic("check_freelist64(1)");
 		}
 		if ( prev == 0 )
 			continue;
@@ -989,12 +992,12 @@ U_INTEGER64 length;
 		if (  prev <= fp->fofs ) {
 			fprintf(stderr,"prv" I64_FORMAT " ofs" I64_FORMAT " typ%x siz" I64_FORMAT "\n",
 				prev,fp->fofs,fp->f.h.type,fp->f.h.size);
-			er_panic("check_freelist(2)");
+			er_panic("check_freelist64(2)");
 		}
 		if ( fp->fofs + fp->f.h.size >= prev ) {
 			fprintf(stderr,"prv" I64_FORMAT " ofs" I64_FORMAT " typ%x siz" I64_FORMAT "\n",
 				prev,fp->fofs,fp->f.h.type,fp->f.h.size);
-			er_panic("check_freelist(3)");
+			er_panic("check_freelist64(3)");
 		}
 	}
 	return 0;
@@ -1328,6 +1331,8 @@ int err_code;
 		size = hh.h32.size;
 		change_endian(hh.h32.size);
 		/*change_endian(size);*/
+		if ( size < sizeof(hh.h32) )
+			goto err;
 		_size = size - sizeof(hh.h32) + sizeof(PN64_HEADER);
 		break;
 	case PHYS_SIZE64:
@@ -1338,6 +1343,8 @@ int err_code;
 			goto err;
 		size = hh.h64.size;
 		change_endian(size);
+		if ( size < sizeof(hh.h64) )
+			goto err;
 		_size = size - sizeof(hh.h64) + sizeof(PN64_HEADER);
 		break;
 	default:
@@ -1345,6 +1352,8 @@ int err_code;
 		size = hh.h16.size;
 		change_endian(hh.h16.size);
 		/*change_endian(size);*/
+		if ( size < sizeof(hh.h16) )
+			goto err;
 		_size = size - sizeof(hh.h16) + sizeof(PN64_HEADER);
 		break;
 	}
