@@ -1,4 +1,4 @@
-/* ver.B.b08(BLAZE) */
+/* ver.B.b16.02(BLAZE) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION ap_version = {
 	"AP.code",
 	"r64status",
-	"(C) 2000-2005 Hirohisa Mori",
-	"ver.B.b08(BLAZE)",
-	2006,4,26,
+	"(C) 2000-2007 Hirohisa Mori",
+	"ver.B.b16.02(BLAZE)",
+	2007,5,27,
 	0,
 	"r64/cr format status viewer ",
 };
