@@ -1,4 +1,4 @@
-/* ver.B.b14(BLAZE) */
+/* ver.B.b16.03(BLAZE) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION ap_version = {
 	"AP.code",
 	"pdbstatus",
-	"(C) 2000-2007 Hirohisa Mori",
-	"ver.B.b14(BLAZE)",
-	2007,2,13,
+	"(C) 2000-2007 Hirohisa Mori and GLOBALBASE Project",
+	"ver.B.b16.03(BLAZE)",
+	2007,6,4,
 	0,
 	"pdb (progressive DB) status viewer ",
 };
